@@ -118,9 +118,16 @@ def get_home_dir():
 
 def get_conf_path(filename=None):
     """Return absolute path for configuration file with specified filename"""
-    # This makes us follow the XDG standard to save our settings
-    # on Linux, as it was requested on Issue 2629
-    if sys.platform.startswith('linux'):
+    # Define conf_dir
+    if PYTEST:
+        import py
+        from _pytest.tmpdir import get_user
+        conf_dir = osp.join(str(py.path.local.get_temproot()),
+                            'pytest-of-{}'.format(get_user()),
+                            SUBFOLDER)
+    elif sys.platform.startswith('linux'):
+        # This makes us follow the XDG standard to save our settings
+        # on Linux, as it was requested on Issue 2629
         xdg_config_home = os.environ.get('XDG_CONFIG_HOME', '')
         if not xdg_config_home:
             xdg_config_home = osp.join(get_home_dir(), '.config')
@@ -129,8 +136,13 @@ def get_conf_path(filename=None):
         conf_dir = osp.join(xdg_config_home, SUBFOLDER)
     else:
         conf_dir = osp.join(get_home_dir(), SUBFOLDER)
+
+    # Create conf_dir
     if not osp.isdir(conf_dir):
-        os.mkdir(conf_dir)
+        if PYTEST:
+            os.makedirs(conf_dir)
+        else:
+            os.mkdir(conf_dir)
     if filename is None:
         return conf_dir
     else:
@@ -280,6 +292,10 @@ def get_interface_language():
     except ValueError:
         locale_language = DEFAULT_LANGUAGE
 
+    # Tests expect English as the interface language
+    if PYTEST:
+        locale_language = DEFAULT_LANGUAGE
+
     language = DEFAULT_LANGUAGE
 
     if locale_language is not None:
@@ -377,8 +393,8 @@ def get_supported_types():
     Note:
     If you update this list, don't forget to update doc/variablexplorer.rst
     """
-    from datetime import date
-    editable_types = [int, float, complex, list, dict, tuple, date
+    from datetime import date, timedelta
+    editable_types = [int, float, complex, list, dict, tuple, date, timedelta
                       ] + list(TEXT_TYPES) + list(INT_TYPES)
     try:
         from numpy import ndarray, matrix, generic
@@ -430,7 +446,7 @@ def running_in_mac_app():
 SAVED_CONFIG_FILES = ('help', 'onlinehelp', 'path', 'pylint.results',
                       'spyder.ini', 'temp.py', 'temp.spydata', 'template.py',
                       'history.py', 'history_internal.py', 'workingdir',
-                      '.projects', '.spyderproject', '.ropeproject',
+                      '.projects', '.spyproject', '.ropeproject',
                       'monitor.log', 'monitor_debug.log', 'rope.log',
                       'langconfig', 'spyder.lock')
 
